using System;
using System.Diagnostics;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using Devv.Core.Utils;
using Devv.Threeter.Twitter;

namespace Devv.Threeter
{
	public partial class FormOptions : Form
	{
		#region Properties

		public Boolean CloseAfterLogin { get; set; }
		public Boolean Loaded { get; set; }

		#endregion

		#region Main methods

		public FormOptions()
		{
			InitializeComponent();

			Loaded = false;

			Icon = Properties.Resources.Threeter;

			cboLanguage.Items.Add(new Language("English", new CultureInfo("en-GB")));
			if (CheckLanguageInstalled("nl")) cboLanguage.Items.Add(new Language("Dutch", new CultureInfo("nl-NL")));
			if (CheckLanguageInstalled("es")) cboLanguage.Items.Add(new Language("Espaol", new CultureInfo("es-ES")));
			if (CheckLanguageInstalled("it")) cboLanguage.Items.Add(new Language("Italiano", new CultureInfo("it-IT"))); 
			if (CheckLanguageInstalled("pt")) cboLanguage.Items.Add(new Language("Portugus", new CultureInfo("pt-BR")));
			
			cboLanguage.SelectedIndex = 0;

			for (Int32 i = 0; i < cboLanguage.Items.Count; i++)
			{
				Language language = (Language)cboLanguage.Items[i];

				if (language.Culture.LCID == Thread.CurrentThread.CurrentUICulture.LCID)
				{
					cboLanguage.SelectedIndex = i;
					i = cboLanguage.Items.Count;
				}
			}

			Loaded = true;
		}

		private void FormOptions_Load(Object sender, EventArgs e)
		{
			LoadSettings();
			LoadAccounts();
			BringToFront();
		}

		private void FormOptions_HelpRequested(Object sender, HelpEventArgs hlpevent)
		{
			Process.Start(GlobalMessages.AppUrl);
			hlpevent.Handled = true;
		}

		private void FormOptions_FormClosing(Object sender, FormClosingEventArgs e)
		{
			Properties.Settings.Default.Save();

			Program.FormPostInstance.RefreshMenuAccounts();
		}

		private void LoadSettings()
		{
			linkLatestVersion.Text = MaintenanceManager.InstalledVersion;

			MaintenanceManager.CheckUpdates();

			chkEnterKeyPosts.Checked = Properties.Settings.Default.EnterKeyPosts;
			chkAutoRun.Checked = Program.AutoRun;
			chkShortenUrl.Checked = Properties.Settings.Default.ShortenUrl;
			chkFriendsUpdates.Checked = Properties.Settings.Default.EnableFriendsUpdates;
			chkMiniMode.Checked = Properties.Settings.Default.MiniMode;
			chkHotKey.Checked = Properties.Settings.Default.EnableHotKey;
			linkHotKey.Text = Properties.Settings.Default.HotKey;

			txtBitLyUser.Text = Properties.Settings.Default.BitLyUser == ShortUrlManager.BITLY_USER ? String.Empty : Properties.Settings.Default.BitLyUser;
			txtBitLyKey.Text = Properties.Settings.Default.BitLyPassword == ShortUrlManager.BITLY_KEY ? String.Empty : Properties.Settings.Default.BitLyPassword;
		}

		#endregion

		#region Accounts

		private void cboAccounts_SelectedIndexChanged(Object sender, EventArgs e)
		{
			if (cboAccounts.SelectedIndex < 0)
			{
				return;
			}

			Program.FormPostInstance.SetCurrentAccount((Account)cboAccounts.SelectedItem);
		}

		private void butEditAccounts_Click(Object sender, EventArgs e)
		{
			Program.ShowFormAccounts(this);
		}

		private void LoadAccounts()
		{
			cboAccounts.DataSource = Program.CurrentAccounts;

			if (Program.CurrentAccounts.Count > 0)
			{
				cboAccounts.SelectedIndex = cboAccounts.FindStringExact(TwitterManager.LoggedAccount.Username);
				cboAccounts.Enabled = true;
			}
			else
			{
				cboAccounts.Enabled = false;
			}
		}

		#endregion

		#region Preferences

		private void chkAutoRun_CheckedChanged(Object sender, EventArgs e)
		{
			try
			{
				if (!chkAutoRun.Checked)
				{
					RegistryUtil.SetAutoRun(GlobalMessages.AppName, String.Empty);
				}
				else
				{
					RegistryUtil.SetAutoRun(GlobalMessages.AppName, Application.ExecutablePath);
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(Messages.ErrorRegistry + Environment.NewLine + ex.Message, Messages.Error, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void chkEnterKeyPosts_CheckedChanged(Object sender, EventArgs e)
		{
			Properties.Settings.Default.EnterKeyPosts = chkEnterKeyPosts.Checked;
		}

		private void chkShortenUrl_CheckedChanged(Object sender, EventArgs e)
		{
			Properties.Settings.Default.ShortenUrl = chkShortenUrl.Checked;
		}

		private void chkFriendsUpdates_CheckedChanged(Object sender, EventArgs e)
		{
			Properties.Settings.Default.EnableFriendsUpdates = chkFriendsUpdates.Checked;

			((FormPost)Application.OpenForms[0]).EnableFriendsUpdates = chkFriendsUpdates.Checked;
		}

		private void chkMiniMode_CheckedChanged(Object sender, EventArgs e)
		{
			Properties.Settings.Default.MiniMode = chkMiniMode.Checked;

			((FormPost)Application.OpenForms[0]).MiniMode = chkMiniMode.Checked;
		}

		#endregion

		#region Hotkey

		private void chkHotKey_CheckedChanged(Object sender, EventArgs e)
		{
			Properties.Settings.Default.EnableHotKey = chkHotKey.Checked;
			linkHotKey.Enabled = chkHotKey.Checked;

			if (chkHotKey.Checked && linkHotKey.Text.Length > 0 && linkHotKey.Text != "...")
			{
				((FormPost)Application.OpenForms[0]).SetHotKey(linkHotKey.Text);
			}
			else
			{
				((FormPost)Application.OpenForms[0]).ClearHotKey();
			}

		}

		private void linkHotKey_LinkClicked(Object sender, LinkLabelLinkClickedEventArgs e)
		{
			using (FormHotKey dialog = new FormHotKey())
			{
				if (dialog.ShowDialog() == DialogResult.OK)
				{
					linkHotKey.Text = dialog.HotKey;
					Properties.Settings.Default.HotKey = dialog.HotKey;

					((FormPost)Application.OpenForms[0]).SetHotKey(dialog.HotKey);
				}
			}
		}

		#endregion

		#region Debug settings

		private void chkDebug_CheckedChanged(object sender, EventArgs e)
		{
			Properties.Settings.Default.Debug = chkDebug.Checked;
			Core.Erroo.Config.DoShowDialog = Properties.Settings.Default.Debug;
		}

		private void chkReportErrors_CheckedChanged(object sender, EventArgs e)
		{
			Properties.Settings.Default.ReportErrors = chkReportErrors.Checked;
		}

		#endregion

		#region Donate

		private void imgDonate_Click(Object sender, EventArgs e)
		{
			Process.Start(GlobalMessages.DonateUrl);
		}

		#endregion

		#region Language selection

		private void cboLanguage_SelectedIndexChanged(Object sender, EventArgs e)
		{
			if (!Loaded)
			{
				return;
			}

			Language language = (Language)cboLanguage.SelectedItem;

			if (language.Culture.Name != Thread.CurrentThread.CurrentUICulture.Name)
			{
				Properties.Settings.Default.Culture = language.Culture.Name;
				Properties.Settings.Default.Save();

				Program.SetCulture(language.Culture, true);

				Application.DoEvents();
				LoadSettings();
			}
		}

		private Boolean CheckLanguageInstalled(String language)
		{
			return Directory.Exists(Path.GetDirectoryName(Application.ExecutablePath) + "\\" + language);
		}

		public class Language
		{
			public String Title { get; set; }
			public CultureInfo Culture { get; set; }

			public Language(String title, CultureInfo culture)
			{
				Title = title;
				Culture = culture;
			}
		}

		#endregion

		#region Version

		private void timerCheckUpdates_Tick(Object sender, EventArgs e)
		{
			if (!String.IsNullOrEmpty(MaintenanceManager.LatestVersion))
			{
				SetLatestVersion();
				timerCheckUpdates.Interval = 720000;
			}

			timerCheckUpdates.Enabled = true;
			timerCheckUpdates.Start();
		}

		private void linkLatestVersion_LinkClicked(Object sender, LinkLabelLinkClickedEventArgs e)
		{
			if (linkLatestVersion.Enabled && !String.IsNullOrEmpty(MaintenanceManager.LatestVersion))
			{
				MaintenanceManager.DownloadLatest();
			}
		}

		private void SetLatestVersion()
		{
			linkLatestVersion.Text = MaintenanceManager.LatestVersion;

			if (MaintenanceManager.HasNewVersion())
			{
				linkHotKey.LinkColor = SystemColors.HotTrack;

				Program.FormPostInstance.ShowTrayBaloon(Messages.VersionAvailable.Replace("#", MaintenanceManager.LatestVersion), GlobalMessages.AppName, "download", ToolTipIcon.Info);
			}
		}

		#endregion

		#region Help

		private void linkIssues_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			Process.Start("http://threeter.codeplex.com/workitem/list/basic");
		}

		private void linkDiscussion_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			Process.Start("http://threeter.codeplex.com/discussions");
		}

		#endregion

		#region Bit.ly settings

		private void txtBitLyUser_TextChanged(object sender, EventArgs e)
		{
			if (String.IsNullOrWhiteSpace(txtBitLyUser.Text))
			{
				Properties.Settings.Default.BitLyUser = ShortUrlManager.BITLY_USER;
			}
			else
			{
				Properties.Settings.Default.BitLyUser = txtBitLyUser.Text.Trim();
			}
		}

		private void txtBitLyKey_TextChanged(object sender, EventArgs e)
		{
			if (String.IsNullOrWhiteSpace(txtBitLyKey.Text))
			{
				Properties.Settings.Default.BitLyPassword = ShortUrlManager.BITLY_KEY;
			}
			else
			{
				Properties.Settings.Default.BitLyPassword = txtBitLyKey.Text.Trim();
			}
		}

		private void linkBitLyReset_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			txtBitLyUser.Text = String.Empty;
			txtBitLyKey.Text = String.Empty;
		}

		#endregion
	}
}